#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <sndfile.h>
#include "SndfileWavUtil.h"

void
generate_wav(const char * pcmfilename, const char * samplewavfilename, const char * outputwavfilename)
{
  char outfilename[2048];
  SNDFILE * outfile ;
  SNDFILE * wavfile ;
  SNDFILE * pcmfile ;
  SF_INFO wavinfo ;
  SF_INFO pcminfo ;
  int buff;
  SF_INSTRUMENT inst ;
  
  memset (&wavinfo, 0, sizeof (wavinfo)) ;

  
  wavfile = sf_open(samplewavfilename, SFM_READ, &wavinfo);

  if (wavfile == NULL){      
    printf ("\nERROR : Not able to open wav file named '%s' : %s/\n", samplewavfilename, sf_strerror (NULL)) ;
    exit (1) ;
  } ;

  printf("WAV format: %x\n", wavinfo.format);

  if (!((wavinfo.format & SF_FORMAT_PCM_16) && (wavinfo.channels == 1) && 
        (wavinfo.format & SF_FORMAT_WAV))){
    printf("\nERROR : .wav file must be SF_FORMAT_PCM_16 in mono\n");
  }

  pcminfo.format = SF_FORMAT_RAW | SF_FORMAT_PCM_16;
  pcminfo.samplerate = wavinfo.samplerate;
  pcminfo.channels = wavinfo.channels;

  pcmfile = sf_open(pcmfilename, SFM_READ, &pcminfo);

  if (pcmfile == NULL){      
    printf ("\nERROR : Not able to open pcm file named '%s' : %s/\n", pcmfilename, sf_strerror (NULL)) ;
    exit (1) ;
  } ;



  outfile = sf_open(outputwavfilename, SFM_WRITE, &wavinfo);

  memset (&inst, 0, sizeof (inst)) ;

  for(int i = SF_STR_FIRST; i <= SF_STR_LAST; i = i + 1) { 
    const char * str =  sf_get_string(wavfile,i);
    if(str != NULL) {
      sf_set_string(outfile,i,str);
    }
  }
  
  if (outfile == NULL){      
    printf ("\nERROR : Not able to create wav file named '%s' : %s/\n", outfilename, sf_strerror (NULL)) ;
    exit (1) ;
  } ;

  while(sf_read_int(pcmfile, &buff, 1) == 1){
    if(sf_write_int(outfile, &buff, 1) != 1){
      printf("\nERROR : unable to write to '%s' : %s/\n", outfilename, sf_strerror(NULL));
    }
  }
  
  sf_close (wavfile) ;
  sf_close (outfile) ;
  sf_close (pcmfile) ;
  
}


void
generate_pcm (const char * wavfilename, const char * pcmfilename)
{
	SNDFILE * wavfile ;
	SNDFILE * pcmfile ;
	SF_INFO wavinfo ;
	SF_INFO pcminfo ;
	int buff;

	memset (&wavinfo, 0, sizeof (wavinfo)) ;
	memset (&pcminfo, 0, sizeof (pcminfo)) ;

	wavfile = sf_open (wavfilename, SFM_READ, &wavinfo) ;

	if (wavfile == NULL){      
	  printf ("\nERROR : Not able to open wav file named '%s' : %s/\n", wavfilename, sf_strerror (NULL)) ;
	  exit (1) ;
	} ;

	pcminfo.format = SF_FORMAT_RAW | SF_FORMAT_PCM_16;
	pcminfo.samplerate = wavinfo.samplerate;
	pcminfo.channels = wavinfo.channels;

	if ((!wavinfo.format & SF_FORMAT_PCM_16) || (!wavinfo.channels == 1)){
	  printf("\nERROR : .wav file must be SF_FORMAT_PCM_16 and mono\n");
	}

	pcmfile = sf_open (pcmfilename, SFM_WRITE, &pcminfo) ;

	if (pcmfile == NULL){      
	  printf ("\nERROR : Not able to create pcm file named '%s' : %s/\n", pcmfilename, sf_strerror (NULL)) ;
	  exit (1) ;
	} ;
	
	while(sf_read_int(wavfile, &buff, 1) == 1){
	  if(sf_write_int(pcmfile, &buff, 1) != 1){
	    printf("\nERROR : unable to write to '%s' : %s/\n", pcmfilename, sf_strerror(NULL));
	  }
	}

	sf_close (wavfile) ;
	sf_close (pcmfile) ;
} 




